;============================================================================
; DBPUB.ASM
;
; Database module public functions.
;============================================================================




;----------------------------------------------------------------------------
DB_Init		proc
;------
;
; Initialise Database module.
;
;
; Entry		n/a
;
;
; Exit		various
;----------------------------------------------------------------------------


; Create Event object and Update mutex.


	invoke	CreateEvent, NULL,		; default security attribute
			FALSE,			; Auto reset
			FALSE,			; Initially not signalled
			addr szUDBEvent		; Event object name

	mov	G_UDBhEvent, eax

	.if	(eax == NULL)			; failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_EventObj,
				0, kErrors, addr ErrorMsgs

	.endif


	invoke	CreateMutex, NULL,		; default security attribute
			NULL,			; no initial owner
			addr szUDBMutex		; Mutex name

	mov	G_UDBhMutex, eax

	.if	(eax == NULL)			; failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_UpdMutex,
				0, kErrors, addr ErrorMsgs

	.endif


; Create update thread in a suspended state.
;
; CreateThread creates a NORMAL priority thread by default, we will
; change this later.


	invoke	CreateThread, NULL, 0, addr UpdateDB, 0,
			CREATE_SUSPENDED, addr UDBThreadId

	mov	hUDBThread, eax

	.if	(eax == NULL)

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_UpdThread,
				0, kErrors, addr ErrorMsgs

	.endif


; Load Group icons and initialise suffix list.


	invoke	LoadGrpSfx


; Open an existing Database / create a new one.


	invoke	File_Exist, addr szDBF		; (returns file size in eax)

	.if	(!CARRY? && eax)

		; Database file found, allocate memory and read it.

		mov	DBSize, eax		; save size
		invoke	Mem_Alloc, eax
		mov	pDataBase, eax

		invoke	File_ReadCl, addr szDBF, pDataBase, DBSize, DBFAttr

		.if	(CARRY?)

			; Read error, abandon it and create a new database.

			xor	eax, eax
			mov	DBSize, eax
			invoke	Mem_DeAlloc, addr pDataBase

			jmp	@@CreateNew

		.endif

		; Build Path, File and Search arrays.

		invoke	BuildArrays

		; Initialise the Search array by searching on everything,
		; subject to initial Group mask - either the default or
		; that in the Tui ini file (loaded above).
		;
		; DB_Search stores the count of files found in LS_FileCnt
		; for Time_Init to retrieve later.

		invoke	DB_Search, I_SRCHRALL, NULL, NULL

		; Database will be stale until the first update.

		or	G_UDBStatus, UDBDbStale

	.else

		@@CreateNew:

		; Database file not found or length is zero, create a
		; new database with the update thread.

		invoke	DialogBoxParamA, G_hTui, ID_ND_DLG, NULL,
				addr NDBDlgHdlr, NULL

		; If the user cancelled then shut down and exit.

		.if	(eax == 0)
			jmp	G_Tui_Exit
		.endif

	.endif

	; Database is open.

	or	DB_OAF, DB_OAF_DBOpen


; Change update thread priority to idle and (re)start it.


	invoke	SetThreadPriority, hUDBThread, THREAD_PRIORITY_IDLE
	invoke	ResumeThread, hUDBThread


; Exit.


	ret



DB_Init		endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_DeInit	proc	uses	ebx esi
;--------
;
; De-Initialise Database module objects.
;
;
; Entry		External:
;
;		DB_OAF		flags indicating objects allocated
;
;		bUDBExit	flag, exit DB update thread
;
;
; Exit		n/a
;
;
; No error handling is done.
;
; .if '&' condition is a bit test on flags == 'test' opcode
;----------------------------------------------------------------------------


	local	szKey[8]		:byte
	local	szStr1[16]		:byte
	local	szStr2[16]		:byte


; Stop the Update thread and release our Event object and Mutex as
; they will not be required after the thread has stopped.


	.if	(hUDBThread)

		; Tell thread to exit and wait until it has.
		;
		; We will give it 5 seconds. If it has not exited then
		; assume it has hung and terminate it.

		mov	bUDBExit, 1

		invoke	WaitForSingleObject, hUDBThread, 5000

		.if	(eax != WAIT_OBJECT_0)
			invoke	TerminateThread, hUDBThread, NULL
		.endif

		invoke	CloseHandle, hUDBThread

	.endif

	.if	(G_UDBhEvent)
		invoke	CloseHandle, G_UDBhEvent
	.endif

	.if	(G_UDBhMutex)
		invoke	CloseHandle, G_UDBhMutex
	.endif


; Save our loaded initialisation parameters.


	; Check we aren't here from the Exception handler, in which case
	; we may not have any suffixes dynamically loaded.

	.if	(pSfxList == 0)
		jmp	@@NoSfxList
	.endif

	; Groups.

	invoke	dw2hex, GrpMask, addr szStr1
	invoke	File_WrIniKey, addr szDBIS_Grp, addr szDBIK_GrpMsk,
			addr szStr1

	; Suffixes.

	invoke	dwtoa, NumSfx, addr szStr1
	invoke	File_WrIniKey, addr szDBIS_Sfx, addr szDBIK_SfxNum,
			addr szStr1

	mov	esi, pSfxList
	xor	ebx, ebx

	.repeat

		; Make Key, just ascii of number.

		invoke	dwtoa, ebx, addr szKey

		; Make Key string from suffix.

		xor	ecx, ecx

		.repeat

			mov	al, [esi + ecx]
			.break	.if (al == 0)
			mov	szStr1[ecx], al
			inc	ecx

		.until	(ecx == 4)

		mov	word ptr szStr1[ecx], ','

		mov	edx, [esi + SFX.FSpec]
		invoke	dw2hex, edx, addr szStr2
		invoke	szCatStr, addr szStr1, addr szStr2

		invoke	File_WrIniKey, addr szDBIS_Sfx, addr szKey,
				 addr szStr1

		add	esi, sizeof SFX
		inc	ebx

	.until	(ebx == NumSfx)

	@@NoSfxList:


; Unload any plugins.
;
; This will do nothing for the first 3 loops as plugins are built-in
; and their initialised pointer is zero (note to self: lazy bastard).


	mov	esi, pPlgList
	mov	ebx, NumPlg

	.repeat

		mov	eax, [esi + PLG.hPLG]

		.if	(eax)
			invoke	FreeLibrary, eax
		.endif

		add	esi, sizeof PLG
		dec	ebx

	.until	(ZERO?)


; Release resources.


	mov	eax, DB_OAF

	.if	(eax & DB_OAF_DBOpen)

		; Save Database to file.

		invoke	File_WriteAll, addr szDBF, pDataBase, DBSize, DBFAttr

	.endif


	; Release icons.

	mov	esi, offset GrpList
	mov	ebx, kGroups

	.repeat

		mov	eax, [esi + GRP.hIcon]

		.if	(eax)
			invoke	DestroyIcon, eax
		.endif

		add	esi, sizeof GRP
		dec	ebx

	.until	(ZERO?)


; Release array, database, suffix list and plugin list memory.


	invoke	Mem_DeAlloc, addr pPathArray
	invoke	Mem_DeAlloc, addr pFileArray
	invoke	Mem_DeAlloc, addr pDataBase
	invoke	Mem_DeAlloc, addr pSfxList
	invoke	Mem_DeAlloc, addr pPlgList


; If we have a temporary file present then delete it.


	invoke	File_Delete, addr szTmpFile


	ret


DB_DeInit	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_GetNfo	proc	uses ebx esi, Request:dword, Ndx:dword, pBuf:dword
;--------
;
; Get various information for a file from the Database, using its index into
; the Search array.
;
;
; Entry,Exit
; ----------
;
; Request:	I_GETRECD - Get pointer to record in Database
;
;			Entry:	pBuf	n/a
;
;			Exit	eax	ptr, File record
;
;		I_GETNAME - Get File name
;
;			Entry:	pBuf	n/a
;
;			Exit:	eax	ptr, File name field
;
;		I_GETPATH - Get File Path
;
;			Entry:	pBuf	n/a
;
;			Exit:	eax	ptr, Path field
;
;		I_GETFULL - Get full path (path + name)
;
;			Entry:	pBuf	ptr, buffer (must be >= MAX_PATH)
;
;			Exit:	pBuf	contains asciiz full path string
;
;		I_GETGRPI - Get File Group ID
;
;			Entry:	pBuf	n/a
;
;			Exit:	eax	Group ID
;
;		I_GETGRPS - Get File Group Name string
;
;			Entry:	pBuf	n/a
;
;			Exit:	eax	pointer to name string
;
;		I_GETICON - Get Icon handle
;
;			Entry:	pBuf	n/a
;
;			Exit:	eax	Icon handle
;
;		I_GETSRCH - Get Searchable status
;
;			Entry:	pBuf	n/a
;
;			Exit:	eax	!0 if Searchable
;
;		I_GETTHUM - Get Thumbnail Handler ID
;
;			Entry:	pBuf	n/a
;
;			Exit:	eax	Thumbnail Handler ID
;
;		I_GETFSFX - Get File Suffix
;
;			Entry:	pBuf	ptr, buffer (must be >= 8 bytes)
;
;			Exit:	pBuf	asciiz suffix string
;					eg: '.123',0,0,0,0 (8 bytes max)
;
;				eax	suffix value
;					eg: 0x02E313233 (big-endian)
;
;		I_GETTMSP - Get TimeStamp as a string
;
;			Entry:	pBuf	ptr, buffer (must be >= 20 bytes)
;
;			Exit:	pBuf	'dd/mm/yyyy hh:mm:ss',0 (20 bytes)
;
; Ndx:		Index into the Search array (0 up)
;
; pBuf:		as for each request above
;
;
; Exit		as above
;----------------------------------------------------------------------------


	local	DummyFrame	:dword		; (for call stack tracing)


; Check index is in range of current Search array.


	mov	eax, Ndx

	.if	(sdword ptr eax < 0 || eax >= LS_FileCnt)


		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_DB_GNxNdx, eax
		endif		;++++++++++++++++++++++++++++++++++++++++++++


		; For now just load with first item so we don't get a crash.

		mov	Ndx, 0

	.endif


; Get pointer to record in the Database, common to all operations.


	mov	eax, Ndx
	shl	eax, ARecMult			; Ndx * array record size
	add	eax, pSrchArray			; + array start address
	mov	esi, [eax]			; = [record pointer]
	mov	eax, esi			; also in eax


; Jump to Request required.


	mov	ebx, Request
	mov	ecx, offset @@JumpTable
	jmp	dword ptr [ecx + ebx*4]

	@@JumpTable:
	dd	offset	@@pRecord
	dd	offset	@@FileName
	dd	offset	@@FilePath
	dd	offset	@@FullPath
	dd	offset	@@FileGrpID
	dd	offset	@@FileGrpName
	dd	offset	@@IconHandle
	dd	offset	@@FileSearch
	dd	offset	@@FileThumb
	dd	offset	@@FileSfx
	dd	offset	@@TimeStamp


@@pRecord:	;------------------------------------------------------------


	jmp	@@ExitOk			; eax has record ptr


@@FileName:	;------------------------------------------------------------


	add	eax, FREC.Nam			; File Name
	jmp	@@ExitOk


@@FilePath:	;------------------------------------------------------------


	mov	eax, [esi + FREC.RSpec]		; Record Specification
	and	eax, RecPNdxMask		; extract PathNdx

	shr	eax, RecPNdxShft-ARecMult	; This replaces these two..
	;shr	eax, RecPNdxShft		; align to b0
	;shl	eax, ARecMult			; * array member size

	add	eax, pPathArray
	mov	eax, [eax]			; pointer to Path
	jmp	@@ExitOk



@@FullPath:	;------------------------------------------------------------


	mov	eax, [esi + FREC.RSpec]		; Record Specification
	and	eax, RecPNdxMask
	shr	eax, RecPNdxShft-ARecMult	; (see above)
	add	eax, pPathArray
	mov	eax, [eax]			; pointer to Path
	invoke	szCopy, eax, pBuf		; copy Path to buffer
	add	esi, FREC.Nam			; Offset to file name
	invoke	szCatStr, pBuf, esi		;  and append to buffer.
	jmp	@@ExitOk


@@FileGrpID:	;------------------------------------------------------------


	mov	eax, [esi + FREC.FSpec]		; File Specification
	and	eax, RecGrpMask			; mask in Group
	jmp	@@ExitOk


@@FileGrpName:	;------------------------------------------------------------


	mov	eax, [esi + FREC.FSpec]		; File Specification
	and	eax, RecGrpMask			; mask in Group
	invoke	DB_GetGrpName, eax
	jmp	@@ExitOk


@@IconHandle:	;------------------------------------------------------------


	mov	eax, [esi + FREC.FSpec]		; File Specification
	and	eax, RecGrpMask			; mask in Group
	dec	eax				; GrpList indexed 0 up
	mov	edx, sizeof GRP
	mul	edx
	add	eax, offset GrpList
	add	eax, GRP.hIcon			; (redundant but necessary)
	mov	eax, [eax]
	jmp	@@ExitOk


@@FileSearch:	;------------------------------------------------------------


	mov	eax, [esi + FREC.FSpec]		; File Specification
	and	eax, RecSrchFlag		; return Searchable status
	jmp	@@ExitOk


@@FileThumb:	;------------------------------------------------------------


	mov	eax, [esi + FREC.FSpec]		; File Specification
	and	eax, RecThumMask		; Return Thumbnail
	shr	eax, RecThumShft		;   Handler ID.
	jmp	@@ExitOk


@@FileSfx:	;------------------------------------------------------------


	mov	eax, [esi + FREC.FSfx]		; File Suffix
	mov	edx, pBuf
	mov	[edx], eax			; (eax = big-endian value)
	xor	ecx, ecx
	mov	[edx + 4], ecx			; (exterminate, exterminate)
	jmp	@@ExitOk


@@TimeStamp:	;------------------------------------------------------------


	; Macro to convert value 0..99 into 2 decimal digits.
	;
	; Entry		ax	value
	;		esi	pointer to output buffer
	;		cl	10
	;		dl	'0'
	;
	; Registers	ax, esi <- esi + 2

	Conv0to99	macro
		div	cl
		add	al, dl			; '0'
		mov	[esi], al		; ms digit
		inc	esi
		add	ah, dl			; '0'
		mov	[esi], ah		; ls digit
		inc	esi
	endm


	mov	ebx, [esi + FREC.TS]		; TimeStamp
	mov	esi, pBuf			; destination

	mov	ecx, 02F0Ah			; ch = '/', cl = 10
	mov	edx, '0'			; dl = '0'

	; Day/Month/.

	mov	eax, ebx
	shr	eax, 17
	and	eax, 01Fh
	Conv0to99
	mov	[esi], ch			; '/'
	inc	esi

	mov	eax, ebx
	shr	eax, 22
	and	eax, 0Fh
	Conv0to99
	mov	[esi], ch			; '/'

	; Year, cannabalised dwtoa.

	push	ebx
	mov	eax, ebx
	shr	eax, 26
	add	eax, 1980
	mov	ecx, 0CCCCCCCDh			; 3435973837
	add	esi, 4				; load in reverse

	.repeat
      		mov	ebx, eax
      		mul	ecx
      		shr	edx, 3
      		mov	eax, edx
      		lea	edx, [edx + edx*4]	; edx*10
      		add	edx, edx
      		sub	ebx, edx
      		add	bl, '0'
      		mov	[esi], bl
      		dec	esi
	.until	(eax == 0)

	pop	ebx
	add	esi, 5				; adjust our pointer

	mov	al, ' '
	mov	[esi], al			; space out
	inc	esi

	; Replace our constants.

	mov	ecx, 03A0Ah			; ch = ':', cl = 10
	mov	edx, '0'			; dl = '0'

	; Hours:Minutes:Seconds

	mov	eax, ebx
	shr	eax, 12
	and	eax, 1Fh
	Conv0to99
	mov	[esi], ch			; ':'
	inc	esi

	mov	eax, ebx
	shr	eax, 6
	and	eax, 3Fh
	Conv0to99
	mov	[esi], ch			; ':'
	inc	esi

	mov	eax, ebx
	and	eax, 3Fh
	Conv0to99
	xor	eax, eax
	mov	[esi], al
	inc	esi

	mov	eax, esi

	; and fall into Exit, ok ...


@@ExitOk:	;------------------------------------------------------------


	clc


@@ExitFail:


	ret


DB_GetNfo	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_Search	proc	uses ebx esi edi, Flags:dword,pText:dword,nText:dword
;--------
;
; Search Database via File array and load Search array with result.
;
;
; Entry		flags:
;
;		I_SRCHCASE	Case sensitive search
;		I_SRCHNAME	Search in File Name
;		I_SRCHNEND	Search in end of File Name
;		I_SRCHDATA	Search in File Data	(see note above)
;		I_SRCHPATH	Search in File Path
;		I_SRCHRALL	Search Array <- File Array
;
;		pText		pointer, search pattern string
;
;		nText		number of characters in string
;
;
; Exit		eax	<>0	eax has Files found, Search array updated
;
;			0	no Files found that match, array unchanged
;
;
; Notes
;
; I_SRCHDATA searchs items ONLY in current Search array!.
;
; NB the 'Search in' flags are mutually exclusive.
;
; Callers should check if files are found and reset the Time display as
; required with Time_SetItem.
;
; Searches are subject to the groups set in the Group mask (GrpMask).
;----------------------------------------------------------------------------


	local	FndCnt			:dword
	local	szIPath[MAX_PATH]	:byte		; full path to item
	local	TmpFile			:MMF		; map file spec
	local	SOfs			:dword		; offset into data


; Search.
;
; (.if '&' condition is a bit test on flags == 'test' opcode)


	mov	esi, pFileArray
	mov	edi, pSrchArray
	xor	ecx, ecx
	mov	FndCnt, ecx
	mov	SOfs, ecx			; zero by default

	cld

	mov	eax, Flags

	.if	(eax == I_SRCHRALL)		; Everything


		; Reload SearchArray from FileArray, subject to Groups.

		.while	(ecx < nFiles)

			; Get record specification and test for Groups.

			mov	eax, esi 		; array record
			mov	ebx, [eax]		; file record ptr
			mov	eax, [ebx + FREC.FSpec]	; file Specification

			and	eax, RecGrpMask
			dec	eax			; bit position
			bt	GrpMask, eax

			.if	(CARRY?)

				; This Group is included, move
				;   FileArray record to SearchArray.

				movsd
				inc	FndCnt

			.else

				; Next record in File Array.

				add	esi, ARecSize

			.endif

			inc	ecx

		.endw


	.elseif	(eax & (I_SRCHNAME + I_SRCHPATH))	; Name or Path


		.while	(ecx < nFiles)

			push	ecx

			; Get record specification and test for Groups.

			mov	eax, esi 			; array record
			mov	ebx, [eax]			; get pointer
			mov	edx, [ebx + FREC.RSpec]		; record Spec
			mov	eax, [ebx + FREC.FSpec]		; file Spec

			and	eax, RecGrpMask
			dec	eax				; bit position
			bt	GrpMask, eax

			.if	(!CARRY?)			; not this Grp
				jmp	@@NextRec
			.endif

			; Get pointer to search target.

			.if	(Flags & I_SRCHNAME)		; File Name

				; If Path Record then ignore.

				.if	(edx & RecPathFlag)
					jmp	@@NextRec
				.else
					add	ebx, FREC.Nam
				.endif

				invoke	StrLen, ebx		; source len

				; If End of Name required then set offset.

				.if	(Flags & I_SRCHNEND)

					mov	edx, eax	; nSrc
					sub	edx, nText	; - nText
					mov	SOfs, edx	; = offset

				.endif

			.elseif	(Flags & I_SRCHPATH)		; File Path

				and	edx, RecPNdxMask	; PathNdx
				shr	edx, RecPNdxShft-ARecMult
				add	edx, pPathArray		; index array
				mov	ebx, [edx]		; [ptr, Path]
				invoke	StrLen, ebx		; source len

			.endif

			; Search for string.
			;
			; ebx points to start of data to search.
			; eax is length of data to search.

			.if	(Flags & I_SRCHCASE)
				invoke	InStrg, SOfs, ebx, eax, pText, nText
			.else
				invoke	InStrgI, SOfs, ebx, eax, pText, nText
			.endif

			.if	(!CARRY?)

				; Item found, move FileArray record
				; to SearchArray.

				movsd
				inc	FndCnt

			.else

				; Next record in File Array.

				@@NextRec:
				add	esi, ARecSize

			.endif

			pop	ecx
			inc	ecx

		.endw


	.elseif	(eax & I_SRCHDATA)		; File Data


		mov	esi, pSrchArray

		.while	(ecx < LS_FileCnt)

			push	ecx

			; Check for known searchable type file.

			invoke	DB_GetNfo, I_GETSRCH, ecx, NULL

			.if	(eax == 0)
				jmp	@@NextFile	; not searchable
			.endif

			; Full path required.

			pop	ecx
			push	ecx

			invoke	DB_GetNfo, I_GETFULL, ecx, addr szIPath

			; Map file into memory.

			lea	eax, szIPath
			mov	TmpFile.pszFile, eax
			mov	TmpFile.nFile, 0
			mov	TmpFile.Access, GENERIC_READ
			mov	TmpFile.Mode, OPEN_EXISTING
			mov	TmpFile.Attr, FAttrAll
			invoke	File_Map, addr TmpFile	; (eax has size)

			jc	@@NextFile		; ignore if error

			; Scan file.

			mov	ebx, TmpFile.pMFile

			.if	(Flags & I_SRCHCASE)
				invoke	InStrg, 0, ebx, eax, pText, nText
			.else
				invoke	InStrgI, 0, ebx, eax, pText, nText
			.endif

			pushf

			; Unmap and close file.

			mov	TmpFile.Mode, FALSE	; don't delete!
			invoke	File_UnMap, addr TmpFile

			; Load result.

			popf

			.if	(!CARRY?)

				; Item found, move SearchArray record
				; down over the last.

				movsd
				inc	FndCnt

			.else

				; Next record in Search Array.

				@@NextFile:
				add	esi, ARecSize

			.endif

			pop	ecx
			inc	ecx

		.endw


	.endif


; If anything found then store the search parameters else leave
; the prior parameters in place.
;
; Also don't store if the search is a Find in Data as upon database
; update this will crash as there is no Search array to find in!.
; Means however that after an update the display is screwed up.


	mov	ebx, FndCnt

	.if	((ebx) && !(Flags & I_SRCHDATA))

		mov	eax, Flags
		mov	LS_Flags, eax

		.if	(eax != I_SRCHRALL)

			mov	ecx, nText
			mov	LS_LenText, ecx
			mov	esi, pText
			mov	edi, offset LS_Text
			rep	movsb
			xor	eax, eax
			mov	[edi], al	; add terminator

		.endif

	.endif


; Return files found.


	mov	LS_FileCnt, ebx			; DB module store
	mov	eax, ebx


	ret


DB_Search	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_GetSearch	proc
;-----------
;
; Get the number of files found in the last search.
;
;
; Entry		n/a
;
;
; Exit		eax		n files
;----------------------------------------------------------------------------


	mov	eax, LS_FileCnt


	ret


DB_GetSearch	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_SetSearch	proc	uses edi esi, Ndx:dword, nItems:dword
;-----------
;
; Set the Search array span by moving all items from Ndx to the end of array,
; down to the start.
;
;
; Entry		Ndx		Item index to start Search array
;
;		nItems		number of items in array
;
;
; Exit		eax		nItems - Ndx
;----------------------------------------------------------------------------


	local	DummyFrame	:dword		; (for call stack tracing)


	mov	eax, nItems
	sub	eax, Ndx

	push	eax

	mov	ecx, eax
	mov	eax, Ndx
	shl	eax, ARecMult		; * bytes per record
	mov	edi, pSrchArray		; beginning of Search array
	mov	esi, edi
	add	esi, eax		; beginning of new data

	cld
	rep	movsd

	pop	eax


	ret


DB_SetSearch	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_GetGrpName	proc	GrpId:dword
;------------
;
; Get a File Group name string.
;
;
; Entry		GrpId		Group ID (NB ID's are 1 based)
;
;
; Exit		eax		pointer to string
;----------------------------------------------------------------------------


	mov	eax, GrpId
	dec	eax				; GrpList is 0 based
	mov	edx, sizeof GRP
	mul	edx
	add	eax, offset GrpList
	add	eax, GRP.szName


	ret


DB_GetGrpName	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_GroupMask	proc	Set:dword, GMask:dword
;-----------
;
; Get/Set the Group mask.
;
;
; Entry		Set		true to Set, false to Get
;
;		GMask		mask
;
;
; Exit		eax		mask
;----------------------------------------------------------------------------


	.if	(Set)
		mov	eax, GMask
		mov	GrpMask, eax
	.else
		mov	eax, GrpMask
	.endif


	ret


DB_GroupMask	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_GetThumData	proc	FTHId:dword, Request:dword
;-------------
;
; Get a File ThumbNail Handler's details.
;
;
; Entry		FTHId		Handler ID
;
;		Request		T_GETFUNC	Get pointer to handler func.
;
;				Exit:		eax has pointer
;
;				T_GETSFX1	Get first or second
;				T_GETSFX2	  file suffix it can handle.
;
;				Exit:		eax has suffix
;
;				T_GETNAME	Get Handler name
;
;				Exit:		eax has pointer to name strg
;
;
; Exit		carry clear	as above
;
;		carry set	Plugin not loaded
;----------------------------------------------------------------------------


; Get pointer to Plugin module's plugin specification.


	mov	eax, FTHId

	; Check we have it loaded.

	.if	(eax >= NumPlg)
		stc
		jmp	@@Exit
	.endif

	shl	eax, PLG_MULT
	mov	edx, pPlgList
	add	edx, eax
	mov	eax, [edx + PLG.pPLG]


; Now what do we want here?.


	mov	edx, Request

	.if	(edx == T_GETFUNC)

		add	eax, PLGSpec.pPLG
		mov	eax, [eax]

	.elseif	(edx == T_GETSFX1)

		add	eax, PLGSpec.Sfx1
		mov	eax, [eax]

	.elseif	(edx == T_GETSFX2)

		add	eax, PLGSpec.Sfx2
		mov	eax, [eax]

	.else	; T_GETNAME

		add	eax, PLGSpec.szName

	.endif

	clc


@@Exit:


	ret


DB_GetThumData	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_SetTypeSpec	proc	uses ebx esi edi,
			DBNdx:dword, GrpListNdx:dword, bText:dword
;-------------
;
; Set a File's Type specification.
;
;
; Entry		DBNdx		File's index into Database
;
;		GrpListNdx	Group List index
;
;		bText		True if this type contains text (searchable)
;
;
; Exit		n/a
;
;
; Note if a file has no plugin but contains text then it is assigned the
; default text thumbnail handler.
;----------------------------------------------------------------------------


	local	szSfx[8]		:byte		; suffix, long form
	local	nSfx			:dword		; suffix, value form
	local	nSpec			:dword		; file spec


; Get File's suffix and scan the Suffix List for it.


	invoke	DB_GetNfo, I_GETFSFX, DBNdx, addr szSfx

	; Eax has suffix value.

	mov	esi, pSfxList
	mov	ecx, NumSfx

	.repeat

		.break	.if (eax == [esi + SFX.Sfx])
		add	esi, sizeof SFX
		dec	ecx

	.until	(ZERO?)


; If it's not here then append it to the end of the List.


	.if	(ecx == 0)

		; Check we have enough room.

		mov	eax, NumSfx
		shl	eax, SFX_MULT

		.if	(eax == nSfxList)

			; No, add another 8 records.

			push eax		; save current size

			add	eax, sizeof SFX	* 8
			mov	nSfxList, eax

			; If we have an error here the original
			; allocation is still ok so the release
			; in DB_DeInit should be ok also.

			invoke	Mem_ReAlloc, addr pSfxList, eax

			; Reset our pointer to the end of the array.

			pop	eax

			mov	esi, pSfxList
			add	esi, eax

		.endif

		; Initialise new suffix and adjust total.

		mov	eax, dword ptr szSfx
		mov	[esi + SFX.Sfx], eax
		inc	NumSfx

	.endif

	mov	nSfx, eax


; Assemble the File Specification and store it to the Suffix List.


	; Thumbnail Handler.
	;
	; Scan Plugin list for suffix. If found then we have a handler.

	xor	ebx, ebx

	.repeat

		invoke	DB_GetThumData, ebx, T_GETSFX1
		.break	.if (eax == nSfx)	; found suffix

		invoke	DB_GetThumData, ebx, T_GETSFX2
		.break	.if (eax == nSfx)	; found suffix

		inc	ebx

	.until	(ebx == NumPlg)

	.if	(ebx == NumPlg)

		; No handler found but if text-like then set
		; to the built-in text handler.

		.if	(bText)
			mov	ebx, FTH_Txt shl RecThumShft
		.else
			xor	ebx, ebx
		.endif

	.else
		shl	ebx, RecThumShft
	.endif

	; Add required Group and searchable flag.

	mov	eax, GrpListNdx			; required Group
	inc	eax
	or	eax, ebx			; Thumbnail ID

	.if	(bText)
		or	eax, RecSrchFlag	; Searchable
	.endif

	mov	[esi + SFX.FSpec], eax
	mov	nSpec, eax


; Update Database.


	mov	esi, pFileArray
	mov	ecx, nFiles

	.repeat

		push	ecx

		; Find files with matching suffix.

		mov	eax, esi 		; array record
		mov	ebx, [eax]		; file record pointer
		mov	edx, ebx
		add	edx, FREC.FSfx
		mov	eax, [edx]		; suffix value
		cmp	eax, nSfx
		jnz	@@NoMatch

		; Match, load new File Spec.

		add	ebx, FREC.FSpec
		mov	eax, nSpec
		mov	[ebx], eax

		@@NoMatch:

		add	esi, ARecSize

		pop	ecx
		dec	ecx

	.until	(ZERO?)


	ret


DB_SetTypeSpec	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_Export	proc	uses ebx esi edi
;--------
;
; Export current Search of database to ascii text file.
;
;
; Entry,Exit	n/a
;
;
; No errorhandling is done.
;----------------------------------------------------------------------------


	local	hOutF		:dword		; handle, output file
	local	Ndx		:dword		; index into Database
	local	szConv[16]	:byte		; dwtoa conversion buffer
	local	szOut[512]	:byte		; line output buffer


; Create our output file.


	invoke	File_Open, addr EXszFileName,
		GENERIC_WRITE,
		CREATE_ALWAYS,			; Overwrite if file exists
		FILE_ATTRIBUTE_NORMAL

	mov	hOutF, eax


; Write out header.


	lea	edi, szOut
	xor	eax, eax			; Preload terminator for
	mov	[edi], eax			;  first szCatStr below.

	; Total Paths.

	invoke	szCatStr, edi, addr EXszPaths
	mov	edx, nPaths
	invoke	dwtoa, edx, addr szConv
	invoke	szCatStr, edi, addr szConv
	invoke	szCatStr, edi, addr G_szNewLine

	; Total Files.

	invoke	szCatStr, edi, addr EXszFiles
	mov	edx, nFiles
	invoke	dwtoa, edx, addr szConv
	invoke	szCatStr, edi, addr szConv
	invoke	szCatStr, edi, addr G_szNewLine
	invoke	szCatStr, edi, addr G_szNewLine

	; Last search parameters.

	invoke	szCatStr, edi, addr EXszSrch
	invoke	szCatStr, edi, addr LS_Text
	invoke	szCatStr, edi, addr G_szNewLine

	invoke	szCatStr, edi, addr EXszGroups

	mov	esi, GrpMask
	xor	ebx, ebx

	.repeat

		inc	ebx
		shr	esi, 1

		.if	(CARRY?)

			invoke	DB_GetGrpName, ebx
			invoke	szCatStr, edi, eax

		.endif

	.until	(esi == 0)

	invoke	szCatStr, edi, addr G_szNewLine

	; Last search results.

	invoke	szCatStr, edi, addr EXszSrchFiles
	mov	edx, LS_FileCnt
	invoke	dwtoa, edx, addr szConv
	invoke	szCatStr, edi, addr szConv

	invoke	szCatStr, edi, addr G_szNewLine
	invoke	szCatStr, edi, addr G_szNewLine
	invoke	szCatStr, edi, addr G_szNewLine

	; Heading.

	invoke	szCatStr, edi, addr EXszHeading

	invoke	szCatStr, edi, addr G_szNewLine
	invoke	szCatStr, edi, addr G_szNewLine
	invoke	File_WriteStrg, hOutF, edi


; And the Database.


	xor	ecx, ecx
	mov	Ndx, ecx

	.while	(ecx < LS_FileCnt)

		push	ecx


		; TimeStamp.

		invoke	DB_GetNfo, I_GETTMSP, Ndx, edi


		; Search Flag and ThumbNail Handler.

		invoke	DB_GetNfo, I_GETSRCH, Ndx, NULL
		.if	(eax)
			invoke	szCatStr, edi, addr EXszFlagSet
		.else
			invoke	szCatStr, edi, addr EXszFlagClr
		.endif

		invoke	DB_GetNfo, I_GETTHUM, Ndx, NULL

		.if	(CARRY?)
			xor	eax, eax	; "None"
		.endif

		invoke	DB_GetThumData, eax, T_GETNAME
		invoke	szCatStr, edi, eax
		invoke	szCatStr, edi, addr G_szTAB


		; Group name.
		;
		; Add a tab if name is < 8 char.

		invoke	DB_GetNfo, I_GETGRPS, Ndx, NULL
		mov	esi, eax
		invoke	StrLen, eax
		push	eax
		invoke	szCatStr, edi, esi
		pop	eax
		.if	(eax < 8)
			invoke	szCatStr, edi, addr G_szTAB
		.endif
		invoke	szCatStr, edi, addr G_szSpc


		; File Suffix.

		invoke	DB_GetNfo, I_GETFSFX, Ndx, addr szConv
		invoke	szCatStr, edi, addr szConv
		invoke	szCatStr, edi, addr G_szTAB


		; File name.
		;
		; Add a tab if name is < 8 char.

		invoke	DB_GetNfo, I_GETNAME, Ndx, NULL
		mov	esi, eax
		invoke	StrLen, eax
		push	eax
		invoke	szCatStr, edi, esi
		invoke	szCatStr, edi, addr G_szTAB
		pop	eax
		.if	(eax < 8)
			invoke	szCatStr, edi, addr G_szTAB
		.endif


		; File path.

		invoke	DB_GetNfo, I_GETPATH, Ndx, NULL
		invoke	szCatStr, edi, eax


		; And write it.

		invoke	szCatStr, edi, addr G_szNewLine

		invoke	File_WriteStrg, hOutF, eax

		inc	Ndx
		pop	ecx
		inc	ecx

	.endw


; Close output file and exit.


	invoke	File_Close, hOutF


	ret


DB_Export	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DB_NDBStop	proc
;---------
;
; Stop the New Database dialog progress bar update.
;
; Stop is required for the fatal exception handler as if there is an error
; somewhere in the NDB dialog handler then we get repeating errors.
;
;
; Entry		n/a
;
;
; Exit		bNDBStop	TRUE
;----------------------------------------------------------------------------


	mov	bNDBStop, 1


	ret


DB_NDBStop	endp
;----------------------------------------------------------------------------




;============================================================================
